const StructJS = require("./");

const MODULES = {EncoderDecoderUTF8: require("./lib/helpers/EncoderDecoderUTF8")};

(() => {
const structJS = new StructJS();
structJS.add`
	typedef int32le i32;
	typedef struct {
		cstring_utf8 myString[256];
		i32 index;
		i32 flags;
		i32 flags2;
		struct {
			float32le x;
			float32le y;
			float32le z;
		} pos;
	} tmp_t;

	typedef tmp_t array_of_tmp_t[1024];
	
	typedef cstring_utf8 str[1];
	
	typedef struct {
		cstring_utf8 char[1];
		cstring_utf8 char2[1];
		i32 dsf;
		uint8 t;
	} array2[1024];
	
	typedef struct {
		uint8 b0;
		uint8 b1;
		uint8 b2;
		uint8 b3;
	} bt;
	
`;
const Marshalling = structJS.compile();
console.log(structJS.code)
const ab = new ArrayBuffer(Marshalling.array_of_tmp_t._size);
const marshal = new Marshalling(ab);
const arrayTmp = new marshal.array_of_tmp_t(0);

const bt = new marshal.bt(0);

const array2 = new marshal.array2(0);
array2.get(0).char = "0";
array2.get(0).char2 = "1";

array2.get(1).char = "2";
array2.get(1).char2 = "34";


console.log( [array2.get(0).char, array2.get(0).char2 ])
console.log( [array2.get(1).char, array2.get(1).char2 ])
console.log( new Uint8Array(ab).subarray(0, 8) )
process.kill(0)
})();

let code = (`
	typedef int32 i32;
	typedef struct {
		float32 x;
		float32 y;
		float32 z;
	} vec3;

	typedef struct {
		float32 x;
		float32 y;
		float32 z;
		float32 w;
	} vec4;

	
	typedef struct {
		vec3 pos;
		vec4 quat;
		
		vec4 matrix[4];
		float32 matrix[4][4];
	} ok;
	
	

	
	/*
	typedef int data2;
	
	typedef struct {
		
		int ident;
		int ident2;
		int ident3;
		struct {
			float x;
			float y;
			float z;
		} pos;
	} cl;
	
	*/
`);


const structJS = new StructJS();
structJS.add(`
	typedef struct {
		float32le x;
		float32le y;
		float32le z;
	} vec3;

	typedef struct {
		float32le x;
		float32le y;
		float32le z;
		float32le w;
	} vec4;

	typedef struct {
		float32 m0;
		float32 m1;
		float32 m2;
		float32 m3;
		float32 m4;
		float32 m5;
		float32 m6;
		float32 m7;
		float32 m8;
		float32 m9;
		float32 m10;
		float32 m11;
		float32 m12;
		float32 m13;
		float32 m14;
		float32 m15;
	} matrix4x4;
	
	
	typedef struct {
		uint8 b;
		cstring_utf8 str[4];
		vec3 pos;
		vec4 quat;
		float32le scale;
		matrix4x4 matrix;
	} model;
	
	typedef model models[1000000];
`);
structJS.compile();


if(0)
(() => {
const array = Array(1e6);
for(let i = 0; i < 1e6; i++)
	array[i] = [Math.random(), Math.random(), Math.random()];

console.time("START")
for(let i = 0; i < 1e6; i++) {
	array[i][0] += Math.random();
	array[i][1] += Math.random();
	array[i][2] += Math.random();
}
console.timeEnd("START")
setInterval(() => 
	console.log(`array[random index].x = ${ array[1e6*Math.random()|0][0] }`), 1e3);
})();

if( 10 )
((marshal) => {
	
	const timeDelta = (label, callback) => {
		let timeDelta = Date.now();
		callback();
		timeDelta = Date.now() - timeDelta;
		console.log(`${label} = ${timeDelta}`);
	}
	
	let ab;
	let types;
	let array;	
	

	timeDelta("init", () => {
		ab = new ArrayBuffer(marshal.models._size);
		types = new marshal(ab);
		array = new types.models(0);
	});
	
	timeDelta("fill", () => {
		array._fill(0);
	});
	
	const firstItem = array.get(0);
	/*
	//firstItem.str = "dataxxxx";
	console.log("firstItem.str", {s: firstItem.str})
	firstItem.str = "o";
	console.log("firstItem.str", {s: firstItem.str})
	firstItem.str = "!";
	console.log("firstItem.str", {s: firstItem.str})
	//firstItem.str = "dataxxxx";
	console.log("firstItem.str", {s: firstItem.str})
	firstItem.str = "!";
	console.log("firstItem.str", {s: firstItem.str})
	*/
	timeDelta("inc", () => {
		const item = array.get(0);
		for(let i = 0; i < 1e6; i++) {
			item.pos.x += Math.random();
			item.pos.y += Math.random();
			item.pos.z += Math.random();
			item._inc();
		}
	});

	timeDelta("_next", () => {
		let item = array.get(0);
		for(let i = 0; i < 1e6; i++) {
			item.pos.x += Math.random();
			item.pos.y += Math.random();
			item.pos.z += Math.random();
			item = item._next();
		}
	});
	
	timeDelta("_for of", () => {
		for(const {pos} of array ) {
			pos.x += Math.random();
			pos.y += Math.random();
			pos.z += Math.random();
		}
	});
	
	setInterval(() => 
		console.log(`array[random index].x = ${ array.get(1e6*Math.random()|0).pos.x }`), 1e6);
	
	const summaryExpected = 1e6*3*3*0.5;
	let summary = 0;
	for(const {pos} of array)
		summary += pos.x + pos.y + pos.z;
	
	console.log(`summary-summaryExpected = ${ Math.round(summary - summaryExpected) }`);
	
})(structJS.marshalling);

console.log( structJS.marshalling )

//export default StructJS;
